﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using LekabMessagingCSharp.LekabMessaging;
using System.Configuration;
using System.ServiceModel.Channels;
using System.Net;
using System.Net.Security;
using System.Runtime.Serialization;
using System.ServiceModel;
using System.Xml;
using System.Security.Cryptography.X509Certificates;

namespace LekabMessagingCSharp
{
    public class MessagingClientProxy
    {
        private messagingv2Client client;
        private static string Username
        {
            get
            {
                return Properties.Settings.Default.Username;
            }
        }

        private static string Password
        {
            get
            {
                return Properties.Settings.Default.Password;
            }
        }
        public MessagingClientProxy()
        {
            InitializeMessagingClient();
        }

        private void InitializeMessagingClient()
        {
            // Create the Web Service Client
            messagingv2Client client = new messagingv2Client();

            // Set the WS-Security Credentials
            client.ClientCredentials.UserName.UserName = Username;
            client.ClientCredentials.UserName.Password = Password;

            this.client = client;
        }

        /**
         * Sends a message and converts FaultException to ErrorDetailException
         */
        public MessageStatuses Send(OutgoingMessage outgoingMessage)
        {
            return client.Send(outgoingMessage);
        }

        /**
         * Gets message statuses and converts FaultException to ErrorDetailException
         */
        public MessageStatuses GetMessageStatus(GetMessageStatusRequest getMessageStatusRequest)
        {
            return client.GetMessageStatus(getMessageStatusRequest);
        }

        /**
         * Gets incoming messages and converts FaultException to ErrorDetailException
         */
        public IncomingMessages GetIncomingMessages(GetIncomingMessagesRequest getIncomingMessagesRequest)
        {
            return client.GetIncomingMessages(getIncomingMessagesRequest);
        }
    }

    public class MessagingClientHelper
    {
        /**
         * Utility method to greate an outgoing SMS message
         */
        public static OutgoingMessage CreateOutgoingSmsMessage(string from, string[] to, string message, string conversationId, bool isTwoway)
        {
            OutgoingMessage outgoingMessage = new OutgoingMessage();
            outgoingMessage.replyable = isTwoway;
            outgoingMessage.sender = from;
            outgoingMessage.recipients = to;
            outgoingMessage.conversationId = conversationId;
            SmsData smsData = new SmsData();
            SmsPayload payload = new SmsPayload();
            UTF8Encoding encoding = new UTF8Encoding();
            payload.message = encoding.GetBytes(message); ;
            smsData.payload = payload;
            Data data = new Data();
            data.Item = smsData;
            outgoingMessage.data = data;
            return outgoingMessage;
        }

        /**
         * Utility method to print message statuses
         */
        public static void PrintMessageStatuses(MessageStatuses messageStatuses)
        {
            Console.WriteLine(GetMessageStatusesString(messageStatuses));
        }
        /**
         * Utility method to print message statuses
         */
        public static string GetMessageStatusesString(MessageStatuses messageStatuses)
        {
            if (messageStatuses == null)
            {
                return "No Statuses received";
            }
            StringBuilder sb = new StringBuilder();
            sb.Append("MessageStatuses:").AppendLine();
            foreach (MessageStatus status in messageStatuses.messageStatus)
            {
                sb.Append("\t").Append("MessageStatus:").AppendLine();
                sb.Append("\t").Append("\t").Append("statusCode = ").Append(status.statusCode).AppendLine();
                sb.Append("\t").Append("\t").Append("statusText = ").Append(status.statusText).AppendLine();
                sb.Append("\t").Append("\t").Append("id = ").Append(status.id).AppendLine();
                sb.Append("\t").Append("\t").Append("recipient = ").Append(status.recipient).AppendLine();
                sb.Append("\t").Append("\t").Append("sender = ").Append(status.sender).AppendLine();
                sb.Append("\t").Append("\t").Append("time = ").Append(status.time).AppendLine();
                sb.Append("\t").Append("\t").Append("conversationId = ").Append(status.conversationId).AppendLine().AppendLine();
            }
            return sb.ToString();
        }
        /**
         * Utility method to print incoming messages
         */
        public static void PrintIncomingMessages(IncomingMessages incomingMessages)
        {
            if (incomingMessages == null || incomingMessages.incomingMessages == null || incomingMessages.incomingMessages.Length == 0)
            {
                Console.WriteLine("No incoming messages received");
            }

            foreach (IncomingMessage incomingMessage in incomingMessages.incomingMessages)
            {
                StringBuilder sb = new StringBuilder();
                bool isSms = false;
                if (incomingMessage.payload.Item is SmsPayload)
                {
                    isSms = true;
                }
                sb.Append("Incoming " + (isSms ? "SMS" : "MMS") + " Message:").AppendLine();

                sb.Append("\t").Append("id = ").Append(incomingMessage.id).AppendLine();
                sb.Append("\t").Append("conversationId = ").Append(incomingMessage.conversationId).AppendLine();
                sb.Append("\t").Append("outgoingMessageId = ").Append(incomingMessage.outgoingMessageId).AppendLine();
                sb.Append("\t").Append("sender = ").Append(incomingMessage.sender).AppendLine();
                sb.Append("\t").Append("recipient = ").Append(incomingMessage.recipient).AppendLine();
                sb.Append("\t").Append("timeStamp = ").Append(incomingMessage.timeStamp).AppendLine();
                if (isSms)
                {
                    SmsPayload payload = (SmsPayload)incomingMessage.payload.Item;
                    UTF8Encoding encoding = new UTF8Encoding();
                    String message = encoding.GetString(payload.message);
                    sb.Append("\t").Append("message = ").Append(message).AppendLine();
                }
                else
                {
                    MmsPayload payload = (MmsPayload)incomingMessage.payload.Item;
                    sb.Append("\t").Append("subject = ").Append(payload.subject).AppendLine();
                }

                Console.WriteLine(sb.ToString());
            }
        }
    }
}
