﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using LekabMessagingCSharp.LekabMessaging;
using System.ServiceModel;

namespace LekabMessagingCSharp
{
    class Program
    {
        static void Main(string[] args)
        {
            // Sender Id for sending one way messages
            string from = "Lekab";
            // An array of recipients
            string[] to = new string[] { "46700123456", "46700123457" };
            string twoWayMessage = "TwoWayMessage from c# with swedish characters åäöÅÄÖ!";
            string oneWayMessage = "OneWayMessage from c# with swedish characters åäöÅÄÖ!";
            Random random = new Random();
            string conversationId = "aConversationId-" + random.Next(1000000, 9999999);

            OutgoingMessage outgoingMessage = MessagingClientHelper.CreateOutgoingSmsMessage(null, to, twoWayMessage, conversationId, true);

            // Send the TwoWay SMS message
            MessageStatuses messageStatuses = null;

            MessagingClientProxy messagingClientProxy = new MessagingClientProxy();
            try
            {
                // Create an outgoing TwoWay SMS message
                Prompt("About to send a TwoWay Message.");
                messageStatuses = messagingClientProxy.Send(outgoingMessage);
            }
            catch (FaultException<ErrorDetails> e)
            {
                PrintErrorDetails(e);
                Prompt("Exit");
                return;
            }
            MessagingClientHelper.PrintMessageStatuses(messageStatuses);

            // Save the message ids for the sent message for later status retrieval
            List<string> ids = new List<string>();
            if (messageStatuses != null && messageStatuses.messageStatus != null && messageStatuses.messageStatus.Length > 0)
            {
                foreach (MessageStatus messageStatus in messageStatuses.messageStatus)
                {
                    ids.Add(messageStatus.id);
                }
            }

            // Create an outgoing OneWay SMS message
            outgoingMessage = MessagingClientHelper.CreateOutgoingSmsMessage(from, to, oneWayMessage, conversationId, false);

            // Send the OneWay SMS message
            try
            {
                Prompt("About to send a OneWay Message."); 
                messageStatuses = messagingClientProxy.Send(outgoingMessage);
            }
            catch (FaultException<ErrorDetails> e)
            {
                PrintErrorDetails(e);
                Prompt("Exit");
                return;
            }
            MessagingClientHelper.PrintMessageStatuses(messageStatuses);

            // Save the message ids for the sent message for later status retrieval
            if (messageStatuses != null && messageStatuses.messageStatus != null && messageStatuses.messageStatus.Length > 0)
            {
                foreach (MessageStatus messageStatus in messageStatuses.messageStatus)
                {
                    ids.Add(messageStatus.id);
                }
            }

            // Get Message status of the sent messages
            GetMessageStatusRequest getMessageStatusRequest = null;
            if (ids.Count > 0)
            {
                Prompt("About to get the MessageStatus of the previously sent messages.");
                getMessageStatusRequest = new GetMessageStatusRequest();
                getMessageStatusRequest.messageIds = ids.ToArray();
                try
                {
                    messageStatuses = messagingClientProxy.GetMessageStatus(getMessageStatusRequest);
                    MessagingClientHelper.PrintMessageStatuses(messageStatuses);
                }
                catch (FaultException<ErrorDetails> e)
                {
                    PrintErrorDetails(e);
                    Prompt("Exit");
                    return;
                }
            }
            else
            {
                PrintMessage("Hmmm... No message status received, That should not happen....");
            }

            // Get New Message status
            Prompt("About to get new MessageStatuses.");
            getMessageStatusRequest = new GetMessageStatusRequest();
            getMessageStatusRequest.maxNumberOfStatuses = 10;
            getMessageStatusRequest.markStatusesRead = false;
            try
            {
                messageStatuses = messagingClientProxy.GetMessageStatus(getMessageStatusRequest);
                MessagingClientHelper.PrintMessageStatuses(messageStatuses);
            }
            catch (FaultException<ErrorDetails> e)
            {
                PrintErrorDetails(e);
                Prompt("Exit");
                return;
            }

            if (messageStatuses != null && messageStatuses.messageStatus != null && messageStatuses.messageStatus.Length > 0)
            {
                ids.Clear();
                foreach (MessageStatus messageStatus in messageStatuses.messageStatus)
                {
                    ids.Add(messageStatus.id);
                }
            }

            // Mark the message statuses as read
            if (ids.Count > 0)
            {
                Prompt("About to mark MessageStatuses as read.");
                getMessageStatusRequest = new GetMessageStatusRequest();
                getMessageStatusRequest.markStatusesRead = true;
                getMessageStatusRequest.messageIds = ids.ToArray();
                try
                {
                    messageStatuses = messagingClientProxy.GetMessageStatus(getMessageStatusRequest);
                    MessagingClientHelper.PrintMessageStatuses(messageStatuses);
                }
                catch (FaultException<ErrorDetails> e)
                {
                    PrintErrorDetails(e);
                    Prompt("Exit");
                    return;
                }
            }

            Prompt("About to get Incoming Messages if any.");

            // Get New Incoming Messages 
            GetIncomingMessagesRequest getIncomingMessagesRequest = new GetIncomingMessagesRequest();
            getIncomingMessagesRequest.markMessagesRead = false;
            getIncomingMessagesRequest.maxNumberOfMessages = 10;
            getIncomingMessagesRequest.retrieveMessages = true;

            IncomingMessages incomingMessages = null;
            try
            {
                incomingMessages = messagingClientProxy.GetIncomingMessages(getIncomingMessagesRequest);
                MessagingClientHelper.PrintIncomingMessages(incomingMessages);
            }
            catch (FaultException<ErrorDetails> e)
            {
                PrintErrorDetails(e);
                Prompt("Exit");
                return;
            }

            // Mark Incoming Messages read

            if (incomingMessages != null && incomingMessages.incomingMessages != null && incomingMessages.incomingMessages.Length > 0)
            {
                Prompt("About to mark the Incoming Messages as read.");

                ids.Clear();
                foreach (IncomingMessage incomingMessage in incomingMessages.incomingMessages)
                {
                    ids.Add(incomingMessage.id);
                }
                getIncomingMessagesRequest = new GetIncomingMessagesRequest();
                getIncomingMessagesRequest.markMessagesRead = true;
                getIncomingMessagesRequest.retrieveMessages = false;
                getIncomingMessagesRequest.messageIds = ids.ToArray();
                try
                {
                    incomingMessages = messagingClientProxy.GetIncomingMessages(getIncomingMessagesRequest);
                }
                catch (FaultException<ErrorDetails> e)
                {
                    PrintErrorDetails(e);
                    Prompt("Exit");
                    return;
                }
                if (incomingMessages == null || incomingMessages.incomingMessages == null || incomingMessages.incomingMessages.Length == 0)
                {
                    PrintMessage("IncomingMessages is empty so everything is in order");
                }
                else
                {
                    PrintMessage("Hmmm... IncomingMessages is not empty.... something went wrong.");
                }
            }
            else
            {
                PrintMessage("No incoming messages received so nothing to mark as read....");
            }

            Prompt("Exit");
        }

        private static void PrintErrorDetails(FaultException<ErrorDetails> e)
        {
            PrintMessage(e.Message);
            ErrorDetails details = e.Detail;
            foreach (ErrorDetail detail in details.errorDetail)
            {
                PrintMessage("ErrorCode: " + detail.errorCode + ", ErrorDescription: " + detail.errorDescription);
            }
        }

        private static void Prompt(string message)
        {
            Console.WriteLine(message);
            Console.WriteLine("Press <Enter> to continue...");
            Console.ReadLine();
        }

        private static void PrintMessage(string message)
        {
            Console.WriteLine(message);
            Console.WriteLine("");
        }
    }
}
